/**
 * \file hardwareSensorOdomRmp400Genom.hpp
 *
 * Header file for getting odometry data from the rmp400 module
 *
 * \date 03/07/2012
 * \author croussil
 *
 * \ingroup rtslam
 */

#ifndef HARDWARE_SENSOR_ODOMRMP400GENOM_HPP_
#define HARDWARE_SENSOR_ODOMRMP400GENOM_HPP_

#include <stdlib.h>
#include <unistd.h>


#include "kernel/jafarMacro.hpp"
#include "rtslam/hardwareSensorAbstract.hpp"

#ifdef HAVE_POSTERLIB
#include "posterLib.h"
#endif


namespace jafar {
namespace rtslam {
namespace hardware {


class HardwareSensorOdomRmp400Genom: public HardwareSensorProprioAbstract
{
	private:
		boost::thread *preloadTask_thread;
		void preloadTask(void);
		
#ifdef HAVE_POSTERLIB
		POSTER_ID posterId;
#endif
		std::string dump_path;
		double last_timestamp;
		kernel::LoggerTask *loggerTask;

		jblas::vec4 calib; ///< lf,rf,lb,rb
		jblas::vec prev_reading;
		double time_stopped;
		bool status400poster;

		void filterOdom(jblas::vec & reading);
		
	public:
		HardwareSensorOdomRmp400Genom(kernel::VariableCondition<int> *condition, unsigned bufferSize,
			Mode mode = mOnline, std::string dump_path = ".", kernel::LoggerTask *loggerTask = NULL);
		void setCalib(jblas::vec4 calib) { this->calib = calib; }

		virtual void start();
		virtual void stop();
		virtual bool join(int timed_ms = -1);
		virtual double getLastTimestamp() { boost::unique_lock<boost::mutex> l(mutex_data); return last_timestamp; }
		
		jblas::ind_array instantValues() { return jmath::ublasExtra::ia_set(1,2); }
		jblas::ind_array incrementValues() { return jmath::ublasExtra::ia_set(1,1); }
};


}}}

#endif // HARDWARE_SENSOR_GPSGENOM_HPP_
